@@ -15,6 +15,10 @@ class ProcInputTypeError(PipenException, TypeError):
 
 
 class ProcInputKeyError(PipenException, KeyError):
+    """When an unsupported input key is provided"""
+
+
+class ProcInputValueError(PipenException, ValueError):
     """When an unsupported input value is provided"""
 
 
@@ -62,5 +66,5 @@ class ConfigurationError(PipenException):
     """When something wrong set as configuration"""
 
 
-class ProcWorkdirConflictException(PipenException):
+class PipenOrProcNameError(PipenException):
     """ "When more than one processes are sharing the same workdir"""
