@@ -1,26 +1,26 @@
 """Main entry module, provide the Pipen class"""
+
 from __future__ import annotations
 
 import asyncio
-from itertools import chain
 from os import PathLike
 from pathlib import Path
-import pprint
-import textwrap
-from typing import Any, ClassVar, Iterable, List, Sequence, Type
+from typing import TYPE_CHECKING, Any, ClassVar, Iterable, List, Sequence, Type
 
 from diot import Diot
 from rich import box
 from rich.panel import Panel
-from rich.table import Table
 from rich.text import Text
-from rich.console import Group
 from simpleconf import ProfileConfig
-from slugify import slugify  # type: ignore
 from varname import varname, VarnameException
+from yunpath import AnyPath
 
 from .defaults import CONFIG, CONFIG_FILES
-from .exceptions import ProcDependencyError, PipenSetDataError
+from .exceptions import (
+    PipenOrProcNameError,
+    ProcDependencyError,
+    PipenSetDataError,
+)
 from .pluginmgr import plugin
 from .proc import Proc
 from .progressbar import PipelinePBar
@@ -28,12 +28,15 @@ from .utils import (
     copy_dict,
     desc_from_docstring,
     get_logpanel_width,
-    get_plugin_context,
+    is_valid_name,
     log_rich_renderable,
     logger,
     pipen_banner,
 )
 
+if TYPE_CHECKING:
+    from xqute.path import PathType
+
 
 class Pipen:
     """The Pipen class provides interface to assemble and run the pipeline
@@ -63,49 +66,56 @@ class Pipen:
     PIPELINE_COUNT: ClassVar[int] = 0
     SETUP: ClassVar[bool] = False
 
-    name: str = None
-    desc: str = None
+    name: str | None = None
+    desc: str | None = None
     outdir: str | PathLike = None
-    starts: List[Proc] = []
-    data: Iterable = None
+    starts: Type[Proc] | List[Type[Proc]] = []
+    data: Iterable | None = None
     # other configs
 
     def __init__(
         self,
-        name: str = None,
-        desc: str = None,
+        name: str | None = None,
+        desc: str | None = None,
         outdir: str | PathLike = None,
         **kwargs,
     ) -> None:
         """Constructor"""
-        self.procs: List[Proc] = None
+        self.procs: List[Type[Proc]] = None
         self.pbar: PipelinePBar = None
         if name is not None:
             self.name = name
         elif self.__class__.name is not None:
             self.name = self.__class__.name
-        elif self.__class__.__name__ != "Pipen":
-            self.name = self.__class__.__name__
         else:
             try:
-                self.name = varname()
+                self.name = varname()  # type: ignore
             except VarnameException:
-                self.name = f"pipen-{self.__class__.PIPELINE_COUNT}"
+                if self.__class__.PIPELINE_COUNT == 0:
+                    self.name = self.__class__.__name__
+                else:
+                    self.name = (
+                        f"{self.__class__.__name__}-" f"{self.__class__.PIPELINE_COUNT}"
+                    )
+
+        if not is_valid_name(self.name):
+            raise PipenOrProcNameError(
+                rf"Invalid pipeline name: {self.name}, expecting '^[\w.-]$'"
+            )
 
         self.desc = (
-            desc
-            or self.__class__.desc
-            or desc_from_docstring(self.__class__)
+            desc or self.__class__.desc or desc_from_docstring(self.__class__, Pipen)
         )
-        self.outdir = Path(
-            outdir
-            or self.__class__.outdir
-            or f"./{slugify(self.name)}_results"
-        ).resolve()
-        self.workdir: Path = None
+        self.outdir: PathType = AnyPath(  # type: ignore
+            outdir or self.__class__.outdir or f"./{self.name}-output"
+        )
+        if isinstance(self.outdir, Path):
+            self.outdir = self.outdir.absolute()
+
+        self.workdir: PathType | None = None
         self.profile: str = "default"
 
-        self.starts: List[Proc] = self.__class__.starts
+        self.starts: List[Type[Proc]] = self.__class__.starts
         if self.starts and not isinstance(self.starts, (tuple, list)):
             self.starts = [self.starts]
 
@@ -117,9 +127,7 @@ class Pipen:
             for key, value in self.__class__.__dict__.items()
             if key in self.config
         }
-        self._kwargs.setdefault("plugin_opts", {}).update(
-            kwargs.pop("plugin_opts", {})
-        )
+        self._kwargs.setdefault("plugin_opts", {}).update(kwargs.pop("plugin_opts", {}))
         self._kwargs.setdefault("template_opts", {}).update(
             kwargs.pop("template_opts", {})
         )
@@ -138,12 +146,14 @@ class Pipen:
             # cyclic imports
             plugin.load_entrypoints()
 
-        plugins = self.config.plugins or self._kwargs.get("plugins", [])
-        self.plugin_context = get_plugin_context(plugins)
+        plugins = self._kwargs.get("plugins", None)
+        if plugins is None:
+            plugins = self.config.plugins
+        self.plugin_context = plugin.plugins_context(plugins)
         self.plugin_context.__enter__()
 
-        # make sure main plugin is enabled
-        plugin.get_plugin("main").enable()
+        # make sure core plugin is enabled
+        plugin.get_plugin("core").enable()
 
         if not self.__class__.SETUP:  # pragma: no cover
             plugin.hooks.on_setup(self.config)
@@ -154,6 +164,9 @@ class Pipen:
         if self.__class__.data is not None:
             self.set_data(*self.__class__.data)
 
+    def __init_subclass__(cls) -> None:
+        cls.PIPELINE_COUNT = 0
+
     async def async_run(self, profile: str = "default") -> bool:
         """Run the processes one by one
 
@@ -164,8 +177,7 @@ class Pipen:
             True if the pipeline ends successfully else False
         """
         self.profile = profile
-        self.workdir = Path(self.config.workdir) / slugify(self.name)
-        self.workdir.mkdir(parents=True, exist_ok=True)
+        self.workdir = AnyPath(str(self.config.workdir)) / self.name  # type: ignore
 
         succeeded = True
         await self._init()
@@ -174,17 +186,17 @@ class Pipen:
         try:
             self.build_proc_relationships()
             self._log_pipeline_info()
+            logger.info("Initializing plugins ...")
             await plugin.hooks.on_start(self)
             for proc in self.procs:
                 self.pbar.update_proc_running()
                 proc_obj = proc(self)  # type: ignore
-                if proc in self.starts and proc.input_data is None:
+                if proc in self.starts and proc.input_data is None:  # type: ignore
                     proc_obj.log(
                         "warning",
-                        "This is a start process, "
-                        "but no 'input_data' specified.",
+                        "This is a start process, but no 'input_data' specified.",
                     )
-                await proc_obj._init()
+                await proc_obj.init()
                 await proc_obj.run()
                 if proc_obj.succeeded:
                     self.pbar.update_proc_done()
@@ -209,7 +221,7 @@ class Pipen:
     def run(
         self,
         profile: str = "default",
-    ) -> int:
+    ) -> bool:
         """Run the pipeline with the given profile
         This is just a sync wrapper for the async `async_run` function using
         `asyncio.run()`
@@ -240,7 +252,7 @@ class Pipen:
         Returns:
             `self` to chain the operations
         """
-        for start, data in zip(self.starts, indata):
+        for start, data in zip(self.starts, indata):  # type: ignore
             if data is None:
                 continue
             if start.input_data is not None:
@@ -276,12 +288,14 @@ class Pipen:
         for proc in procs:
             if isinstance(proc, (list, tuple)):
                 self.set_starts(*proc, clear=False)
-            elif proc not in self.starts:
+            elif not isinstance(proc, type) or not issubclass(proc, Proc):
+                raise ProcDependencyError(
+                    f"{proc!r} is not a subclass of 'pipen.Proc'."
+                )
+            elif proc not in self.starts:  # type: ignore
                 self.starts.append(proc)  # type: ignore
             else:
-                raise ProcDependencyError(
-                    f"{proc} is already a start process."
-                )
+                raise ProcDependencyError(f"{proc} is already a start process.")
         return self
 
     # In case people forget the "s"
@@ -289,110 +303,50 @@ class Pipen:
 
     def _log_pipeline_info(self) -> None:
         """Print the information of the pipeline"""
-        items = (
-            [
-                Panel(
-                    self.desc,
-                    box=box.Box(
-                        "    \n    \n    \n    \n    \n    \n    \n────\n",
-                    ),
-                    padding=(0, 0),
-                )
-            ]
-            if self.desc
-            else []
-        )
-
-        items_table = Table.grid(padding=(0, 1), pad_edge=True)
-        items.append(items_table)  # type: ignore
-        enabled_plugins = [
-            "{name}{version}".format(
-                name=name,
-                version=(f"-{plg.__version__}" if plg.version else ""),
-            )
-            for name, plg in plugin.get_enabled_plugins().items()
-        ]
-        config = ProfileConfig.detach(self.config)
-        for key, value in chain(
-            zip(
-                ["# procs", "plugins", "profile", "outdir"],
-                [
-                    str(len(self.procs)),
-                    enabled_plugins,
-                    self.profile,
-                    self.outdir,
-                ],
-            ),
-            sorted(
-                (key, val)
-                for key, val in config.items()
-                if not key.endswith("_opts")
-            ),
-            (
-                (
-                    "plugin_opts",
-                    pprint.pformat(config.plugin_opts, indent=1),
-                ),
-                (
-                    "scheduler_opts",
-                    pprint.pformat(config.scheduler_opts, indent=1),
-                ),
-                (
-                    "template_opts",
-                    pprint.pformat(
-                        {
-                            key: (
-                                {
-                                    ckey: textwrap.shorten(
-                                        str(cval),
-                                        width=30 - len(key),
-                                        placeholder=" …",
-                                    )
-                                    for ckey, cval in chain(
-                                        list(val.items())[:3],
-                                        []
-                                        if len(val) <= 3
-                                        else [("...", "...")],
-                                    )
-                                }
-                                if isinstance(val, dict)
-                                else val
-                            )
-                            for key, val in config.template_opts.items()
-                        },
-                        indent=1,
-                        # sort_dicts=False,
-                    ),
-                ),
-            ),
-        ):
-            items_table.add_row(
-                Text.assemble((key, "scope.key")),
-                Text.assemble(("=", "scope.equals")),
-                Text(str(value), overflow="fold"),
-            )
-
         logger.info("")
+        # Pipeline line and description
         log_rich_renderable(
             Panel(
-                Group(*items),
-                title=self.name.upper(),
+                self.desc or Text(self.name.upper(), justify="center"),
                 width=get_logpanel_width(),
-                box=box.Box(
-                    "╭═┬╮\n"
-                    "║ ║║\n"
-                    "├═┼┤\n"
-                    "║ ║║\n"
-                    "├═┼┤\n"
-                    "├═┼┤\n"
-                    "║ ║║\n"
-                    "╰═┴╯\n"
-                ),
-                padding=(0, 1),
+                # padding=(0, 1),
+                box=box.DOUBLE_EDGE,
+                title=self.name.upper() if self.desc else None,
             ),
-            None,
+            "magenta",
             logger.info,
         )
+        fmt = "[bold][magenta]%-16s:[/magenta][/bold] %s"
+        enabled_plugins = (
+            "{name} [cyan]{version}[/cyan]".format(
+                name=name,
+                version=(f"v{plg.version}" if plg.version else ""),
+            )
+            for name, plg in plugin.get_enabled_plugins().items()
+            if name != "core"
+        )
+        for i, plug in enumerate(enabled_plugins):
+            logger.info(fmt, "plugins" if i == 0 else "", plug)
+        logger.info(fmt, "# procs", len(self.procs))
+        logger.info(fmt, "profile", self.profile)
+        logger.info(fmt, "outdir", self.outdir)
+        logger.info(fmt, "cache", self.config.cache)
+        logger.info(fmt, "dirsig", self.config.dirsig)
+        logger.info(fmt, "error_strategy", self.config.error_strategy)
+        logger.info(fmt, "forks", self.config.forks)
+        logger.info(fmt, "lang", self.config.lang)
+        logger.info(fmt, "loglevel", self.config.loglevel)
+        logger.info(fmt, "num_retries", self.config.num_retries)
+        logger.info(fmt, "scheduler", self.config.scheduler)
+        logger.info(fmt, "submission_batch", self.config.submission_batch)
+        logger.info(fmt, "template", self.config.template)
+        logger.info(fmt, "workdir", self.workdir)
+        for i, (key, val) in enumerate(self.config.plugin_opts.items()):
+            logger.info(fmt, "plugin_opts" if i == 0 else "", f"{key}={val}")
+        for i, (key, val) in enumerate(self.config.scheduler_opts.items()):
+            logger.info(fmt, "scheduler_opts" if i == 0 else "", f"{key}={val}")
+        for i, (key, val) in enumerate(self.config.template_opts.items()):
+            logger.info(fmt, "template_opts" if i == 0 else "", f"{key}={val}")
 
     async def _init(self) -> None:
         """Compute the configurations for the pipeline based on the priorities
@@ -411,9 +365,7 @@ class Pipen:
             *CONFIG_FILES,
             ignore_nonexist=True,
         )
-        self.config = ProfileConfig.use_profile(
-            config, self.profile, copy=True
-        )
+        self.config = ProfileConfig.use_profile(config, self.profile, copy=True)
 
         # configs from files and CONFIG are loaded
         # allow plugins to change the default configs
@@ -421,12 +373,15 @@ class Pipen:
         # Then load the extra configurations passed from __init__(**kwargs)
         # Make sure dict options get inherited
         self.config.template_opts.update(self._kwargs.pop("template_opts", {}))
-        self.config.scheduler_opts.update(
-            self._kwargs.pop("scheduler_opts", {})
-        )
+        self.config.scheduler_opts.update(self._kwargs.pop("scheduler_opts", {}))
         self.config.plugin_opts.update(self._kwargs.pop("plugin_opts", {}))
         self.config.update(self._kwargs)
 
+        if "workdir" in self._kwargs:
+            self.workdir = AnyPath(self._kwargs["workdir"]) / self.name  # type: ignore
+
+        self.workdir.mkdir(parents=True, exist_ok=True)
+
     def build_proc_relationships(self) -> None:
         """Build the proc relationships for the pipeline"""
         if self.procs:
@@ -439,22 +394,22 @@ class Pipen:
             )
 
         # build proc relationships
-        self.procs = self.starts[:]
-        nexts = set(
-            sum((proc.nexts or [] for proc in self.procs), [])  # type: ignore
-        )
+        # Allow starts to be set as a tuple
+        self.procs = list(self.starts)  # type: ignore
+        nexts = set(sum((proc.nexts or [] for proc in self.procs), []))  # type: ignore
         logger.debug("")
         logger.debug("Building process relationships:")
         logger.debug("- Start processes: %s", self.procs)
         while nexts:
             logger.debug("- Next processes: %s", nexts)
             # pick up one that can be added to procs
-            for proc in sorted(
-                nexts, key=lambda prc: (prc.order or 0, prc.name)
-            ):
+            for proc in sorted(nexts, key=lambda prc: (prc.order or 0, prc.name)):
                 if proc in self.procs:
-                    raise ProcDependencyError(
-                        f"Cyclic dependency: {proc.name}"
+                    raise ProcDependencyError(f"Cyclic dependency: {proc.name}")
+
+                if proc.name in [p.name for p in self.procs]:
+                    raise PipenOrProcNameError(
+                        f"'{proc.name}' is already used by another process."
                     )
 
                 # Add proc to self.procs if all their requires
@@ -477,3 +432,37 @@ class Pipen:
                     )
 
         self.pbar = PipelinePBar(len(self.procs), self.name.upper())
+
+
+def run(
+    name: str,
+    starts: Type[Proc] | List[Type[Proc]],
+    data: Iterable = None,
+    *,
+    desc: str = None,
+    outdir: PathType | None = None,
+    profile: str = "default",
+    **kwargs,
+) -> bool:
+    """Shortcut to run a pipeline
+
+    Args:
+        name: The name of the pipeline
+        starts: The start processes
+        data: The input data for the start processes
+        desc: The description of the pipeline
+        outdir: The output directory of the results
+        profile: The profile to use
+        **kwargs: Other options pass to Pipen to create the pipeline
+
+    Returns:
+        True if the pipeline ends successfully else False
+    """
+    pipeline = Pipen(
+        name=name,
+        desc=desc,
+        outdir=outdir,
+        **kwargs,
+    )
+    pipeline.set_starts(starts).set_data(data)
+    return pipeline.run(profile)
