@@ -1,10 +1,12 @@
 """Provides the process class: Proc"""
+
 from __future__ import annotations
 
 import asyncio
 import inspect
-from abc import ABC, ABCMeta
 import logging
+from abc import ABC, ABCMeta
+from functools import cached_property
 from os import PathLike
 from pathlib import Path
 from typing import (
@@ -17,13 +19,11 @@ from typing import (
     TYPE_CHECKING,
 )
 
-# Slow down the import, try do it at runtime
-# import pandas
 from diot import Diot
 from rich import box
 from rich.panel import Panel
-from slugify import slugify  # type: ignore
 from varname import VarnameException, varname
+from yunpath import AnyPath
 from xqute import JobStatus, Xqute
 
 from .defaults import ProcInputType
@@ -31,19 +31,19 @@ from .exceptions import (
     ProcInputKeyError,
     ProcInputTypeError,
     ProcScriptFileNotFound,
-    ProcWorkdirConflictException,
+    PipenOrProcNameError,
 )
 from .pluginmgr import plugin
 from .scheduler import get_scheduler
 from .template import Template, get_template_engine
 from .utils import (
     brief_list,
-    cached_property,
     copy_dict,
     desc_from_docstring,
     get_logpanel_width,
     ignore_firstline_dedent,
     is_subclass,
+    is_valid_name,
     log_rich_renderable,
     logger,
     make_df_colnames_unique_inplace,
@@ -55,7 +55,7 @@ from .utils import (
 
 if TYPE_CHECKING:  # pragma: no cover
     from .pipen import Pipen
-    from .procgroup import ProcGroup
+    from .scheduler import Scheduler
 
 
 class ProcMeta(ABCMeta):
@@ -89,7 +89,6 @@ class ProcMeta(ABCMeta):
 
 
 class Proc(ABC, metaclass=ProcMeta):
-
     """The abstract class for processes.
 
     It's an abstract class. You can't instantise a process using it directly.
@@ -108,6 +107,7 @@ class Proc(ABC, metaclass=ProcMeta):
             the docstring by default.
         envs: The arguments that are job-independent, useful for common options
             across jobs.
+        envs_depth: How deep to update the envs when subclassed.
         cache: Should we detect whether the jobs are cached?
         dirsig: When checking the signature for caching, whether should we walk
             through the content of the directory? This is sometimes
@@ -156,6 +156,7 @@ class Proc(ABC, metaclass=ProcMeta):
     name: str = None
     desc: str = None
     envs: Mapping[str, Any] = None
+    envs_depth: int = None
     cache: bool = None
     dirsig: bool = None
     export: bool = None
@@ -179,7 +180,10 @@ class Proc(ABC, metaclass=ProcMeta):
     nexts: Sequence[Type[Proc]] = None
     output_data: Any = None
     workdir: PathLike = None
-    __procgroup__: ProcGroup | None = None
+    # metadata that marks the process
+    # Can also be used for plugins
+    # It's not inheirted
+    __meta__: Mapping[str, Any] = None
 
     @classmethod
     def from_proc(
@@ -188,6 +192,7 @@ class Proc(ABC, metaclass=ProcMeta):
         name: str = None,
         desc: str = None,
         envs: Mapping[str, Any] = None,
+        envs_depth: int = None,
         cache: bool = None,
         export: bool = None,
         error_strategy: str = None,
@@ -209,6 +214,7 @@ class Proc(ABC, metaclass=ProcMeta):
             desc: The new description of the process
             envs: The arguments of the process, will overwrite parent one
                 The items that are specified will be inherited
+            envs_depth: How deep to update the envs when subclassed.
             cache: Whether we should check the cache for the jobs
             export: When True, the results will be exported to
                 `<pipeline.outdir>`
@@ -237,8 +243,8 @@ class Proc(ABC, metaclass=ProcMeta):
         """
         if not name:
             try:
-                name = varname()
-            except VarnameException as vexc:
+                name = varname()  # type: ignore
+            except VarnameException as vexc:  # pragma: no cover
                 raise ValueError(
                     "Process name cannot be detected from assignment, "
                     "pass one explicitly to `Proc.from_proc(..., name=...)`"
@@ -257,6 +263,7 @@ class Proc(ABC, metaclass=ProcMeta):
         for key in (
             "desc",
             "envs",
+            "envs_depth",
             "cache",
             "forks",
             "order",
@@ -272,36 +279,47 @@ class Proc(ABC, metaclass=ProcMeta):
 
         kwargs["__doc__"] = proc.__doc__
         out = type(name, (proc,), kwargs)
-        out.__procgroup__ = None
         return out
 
     def __init_subclass__(cls) -> None:
         """Do the requirements inferring since we need them to build up the
         process relationship
         """
-        # Use __mro__?
-        parent = cls.__bases__[-1]
+        base = [
+            mro
+            for mro in cls.__mro__
+            if issubclass(mro, Proc) and mro is not Proc and mro is not cls
+        ]
+        parent = base[0] if base else None
         # cls.requires = cls._compute_requires()
         # triggers cls.__setattr__() to compute requires
         cls.nexts = []
         cls.requires = cls.requires
 
-        if cls.name is None or cls.name == parent.name:
+        if cls.name is None or (parent and cls.name == parent.name):
             cls.name = cls.__name__
 
-        if cls.__doc__ is None:
-            cls.__doc__ = (
-                parent.__doc__
-                if parent is not Proc
-                else "Undescribed process."
+        if not is_valid_name(cls.name):
+            raise PipenOrProcNameError(
+                f"{cls.name} is not a valid process name, expecting " r"'^[\w.-]+$'"
             )
 
-        cls.envs = update_dict(parent.envs, cls.envs)
-        cls.plugin_opts = update_dict(parent.plugin_opts, cls.plugin_opts)
-        cls.scheduler_opts = update_dict(
-            parent.scheduler_opts, cls.scheduler_opts
+        envs = update_dict(
+            parent.envs if parent else None,
+            cls.envs,
+            depth=0 if not parent or parent.envs_depth is None else parent.envs_depth,
         )
-        cls.__procgroup__ = None
+        # So values can be accessed like Proc.envs.a.b
+        cls.envs = envs if isinstance(envs, Diot) else Diot(envs or {})
+        cls.plugin_opts = update_dict(
+            parent.plugin_opts if parent else None,
+            cls.plugin_opts,
+        )
+        cls.scheduler_opts = update_dict(
+            parent.scheduler_opts if parent else {},
+            cls.scheduler_opts,
+        )
+        cls.__meta__ = {"procgroup": None}
 
     def __init__(self, pipeline: Pipen = None) -> None:
         """Constructor
@@ -316,18 +334,17 @@ class Proc(ABC, metaclass=ProcMeta):
 
         self.pbar = None
         self.jobs: List[Any] = []
-        self.xqute = None
-        self.__class__.workdir = Path(self.pipeline.workdir) / slugify(
-            self.name
+        self.xqute: Xqute | None = None
+        self.__class__.workdir = (
+            AnyPath(self.pipeline.workdir) / self.name  # type: ignore
         )
-
         # plugins can modify some default attributes
-        plugin.hooks.on_proc_init(self)
+        plugin.hooks.on_proc_create(self)
 
         # Compute the properties
         # otherwise, the property can be accessed directly from class vars
         if self.desc is None:
-            self.desc: str = desc_from_docstring(self.__class__)
+            self.desc: str = desc_from_docstring(self.__class__, Proc)
 
         if self.export is None:
             self.export = bool(not self.nexts)
@@ -349,65 +366,54 @@ class Proc(ABC, metaclass=ProcMeta):
 
         # input
         self.input = self._compute_input()  # type: ignore
-        plugin.hooks.on_proc_input_computed(self)
         # output
         self.output = self._compute_output()
+        plugin.hooks.on_proc_input_computed(self)
         # scheduler
-        self.scheduler = get_scheduler(  # type: ignore
+        self.scheduler: Type[Scheduler] = get_scheduler(  # type: ignore
             self.scheduler or self.pipeline.config.scheduler
         )
         # script
         self.script = self._compute_script()  # type: ignore
-
-        # check if it's the same proc using the workdir
-        # since the directory name is slugified
-        proc_name_file = self.workdir / "proc.name"  # type: ignore
-        if (
-            proc_name_file.is_file()
-            and proc_name_file.read_text() != self.name
-        ):
-            raise ProcWorkdirConflictException(
-                "Workdir name is conflicting with process "
-                f"{proc_name_file.read_text()!r}, use a differnt pipeline "
-                "workdir or a different process name."
-            )
         self.workdir.mkdir(exist_ok=True)
-        proc_name_file.write_text(self.name)
 
         if self.submission_batch is None:
             self.submission_batch = self.pipeline.config.submission_batch
 
-    async def _init(self) -> None:
+    async def init(self) -> None:
         """Init all other properties and jobs"""
         import pandas
 
-        scheduler_opts = (
-            copy_dict(self.pipeline.config.scheduler_opts, 2) or {}
-        )
+        scheduler_opts = copy_dict(self.pipeline.config.scheduler_opts, 2) or {}
         scheduler_opts.update(self.scheduler_opts or {})
+
         self.xqute = Xqute(
             self.scheduler,
-            job_metadir=self.workdir,
-            job_submission_batch=self.submission_batch,
-            job_error_strategy=self.error_strategy
-            or self.pipeline.config.error_strategy,
-            job_num_retries=self.pipeline.config.num_retries
-            if self.num_retries is None
-            else self.num_retries,
-            scheduler_forks=self.forks or self.pipeline.config.forks,
-            scheduler_jobprefix=self.name,
-            **scheduler_opts,
+            workdir=self.workdir,
+            submission_batch=self.submission_batch,
+            error_strategy=self.error_strategy or self.pipeline.config.error_strategy,
+            num_retries=(
+                self.pipeline.config.num_retries
+                if self.num_retries is None
+                else self.num_retries
+            ),
+            forks=self.forks or self.pipeline.config.forks,
+            jobname_prefix=self.name,
+            scheduler_opts=scheduler_opts,
         )
+        self.xqute.scheduler.post_init(self)
         # for the plugin hooks to access
         self.xqute.proc = self
 
+        await plugin.hooks.on_proc_init(self)
         await self._init_jobs()
-        self.__class__.output_data = pandas.DataFrame(
-            (job.output for job in self.jobs)
-        )
+        self.__class__.output_data = pandas.DataFrame((job.output for job in self.jobs))
 
     def gc(self):
         """GC process for the process to save memory after it's done"""
+        del self.xqute.jobs[:]
+        self.xqute.jobs = []
+
         del self.xqute
         self.xqute = None
 
@@ -453,10 +459,7 @@ class Proc(ABC, metaclass=ProcMeta):
         for job in self.jobs:
             if await job.cached:
                 cached_jobs.append(job.index)
-                self.pbar.update_job_submitted()
-                self.pbar.update_job_running()
-                self.pbar.update_job_succeeded()
-                job.status = JobStatus.FINISHED
+                await plugin.hooks.on_job_cached(job)
             else:
                 await self.xqute.put(job)
         if cached_jobs:
@@ -465,11 +468,11 @@ class Proc(ABC, metaclass=ProcMeta):
         self.pbar.done()
         await plugin.hooks.on_proc_done(
             self,
-            False
-            if not self.succeeded
-            else "cached"
-            if len(cached_jobs) == self.size
-            else True,
+            (
+                False
+                if not self.succeeded
+                else "cached" if len(cached_jobs) == self.size else True
+            ),
         )
 
     # properties
@@ -535,14 +538,11 @@ class Proc(ABC, metaclass=ProcMeta):
         Args:
             config: The pipeline configuration
         """
-
         for i in range(self.input.data.shape[0]):
-            job = self.scheduler.job_class(i, "", self.workdir)
+            job = self.xqute.scheduler.create_job(i, "")
             self.jobs.append(job)
 
-        await asyncio.gather(
-            *(self._init_job(i) for i in range(self.submission_batch))
-        )
+        await asyncio.gather(*(self._init_job(i) for i in range(self.submission_batch)))
 
     def _compute_input(self) -> Mapping[str, Mapping[str, Any]]:
         """Calculate the input based on input and input data
@@ -595,15 +595,17 @@ class Proc(ABC, metaclass=ProcMeta):
             out.data = pandas.concat(
                 (req.output_data for req in self.requires),  # type: ignore
                 axis=1,
-            ).fillna(method="ffill")
+            ).ffill()
 
         make_df_colnames_unique_inplace(out.data)
 
         # try match the column names
         # if none matched, use the first columns
-        rest_cols = out.data.columns.difference(out.type, False)
+        # rest_cols = out.data.columns.difference(out.type, False)
+        rest_cols = [col for col in out.data.columns if col not in out.type]
         len_rest_cols = len(rest_cols)
-        matched_cols = out.data.columns.intersection(out.type)
+        # matched_cols = out.data.columns.intersection(out.type)
+        matched_cols = [col for col in out.data.columns if col in out.type]
         needed_cols = [col for col in out.type if col not in matched_cols]
         len_needed_cols = len(needed_cols)
 
@@ -662,51 +664,50 @@ class Proc(ABC, metaclass=ProcMeta):
                 )
                 script_file = Path(inspect.getfile(base)).parent / script_file
             if not script_file.is_file():
-                raise ProcScriptFileNotFound(
-                    f"No such script file: {script_file}"
-                )
+                raise ProcScriptFileNotFound(f"No such script file: {script_file}")
             script = script_file.read_text()
 
-        script = ignore_firstline_dedent(script)
+        self.script = ignore_firstline_dedent(script)
         if not self.lang:
-            self.lang = get_shebang(script)
+            self.lang = get_shebang(self.script)
 
-        return self.template(script, **self.template_opts)  # type: ignore
+        plugin.hooks.on_proc_script_computed(self)
+        return self.template(self.script, **self.template_opts)  # type: ignore
 
     def _log_info(self):
         """Log some basic information of the process"""
         title = (
-            f"{self.__procgroup__.name}/{self.name}"
-            if self.__procgroup__
+            f"{self.__meta__['procgroup'].name}/{self.name}"
+            if self.__meta__["procgroup"]
             else self.name
         )
         panel = Panel(
             self.desc or "Undescribed",
             title=title,
-            box=box.Box(
-                "╭═┬╮\n"
-                "║ ║║\n"
-                "├═┼┤\n"
-                "║ ║║\n"
-                "├═┼┤\n"
-                "├═┼┤\n"
-                "║ ║║\n"
-                "╰═┴╯\n"
-            )
-            if self.export
-            else box.ROUNDED,
+            box=(
+                box.Box(
+                    "╭═┬╮\n"
+                    "║ ║║\n"
+                    "├═┼┤\n"
+                    "║ ║║\n"
+                    "├═┼┤\n"
+                    "├═┼┤\n"
+                    "║ ║║\n"
+                    "╰═┴╯\n"
+                )
+                if self.export
+                else box.ROUNDED
+            ),
             width=get_logpanel_width(),
         )
 
         logger.info("")
         log_rich_renderable(panel, "cyan", logger.info)
-        self.log("info", "Workdir: %r", str(self.workdir))
+        self.log("info", "workdir: %r", str(self.workdir))
         self.log(
             "info",
             "[yellow]<<<[/yellow] %s",
-            [proc.name for proc in self.requires]
-            if self.requires
-            else "[START]",
+            [proc.name for proc in self.requires] if self.requires else "[START]",
         )
         self.log(
             "info",
